@@ -1,4 +1,4 @@
-/* $OpenBSD: enc.c,v 1.12 2017/01/20 08:57:12 deraadt Exp $ */
+/* $OpenBSD: enc.c,v 1.20 2019/04/01 16:06:54 jsing Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -56,7 +56,6 @@
  * [including the GNU Public Licence.]
  */
 
-#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -89,6 +88,7 @@ static struct {
 	char *hkey;
 	char *hsalt;
 	char *inf;
+	int iter;
 	char *keyfile;
 	char *keystr;
 	char *md;
@@ -97,6 +97,7 @@ static struct {
 	int olb64;
 	char *outf;
 	char *passarg;
+	int pbkdf2;
 	int printkey;
 	int verbose;
 } enc_config;
@@ -175,6 +176,13 @@ static struct option enc_options[] = {
 		.type = OPTION_ARG,
 		.opt.arg = &enc_config.inf,
 	},
+	{
+		.name = "iter",
+		.argname = "iterations",
+		.desc = "Specify iteration count and force use of PBKDF2",
+		.type = OPTION_ARG_INT,
+		.opt.value = &enc_config.iter,
+	},
 	{
 		.name = "iv",
 		.argname = "IV",
@@ -253,6 +261,12 @@ static struct option enc_options[] = {
 		.type = OPTION_ARG,
 		.opt.arg = &enc_config.passarg,
 	},
+	{
+		.name = "pbkdf2",
+		.desc = "Use the pbkdf2 key derivation function",
+		.type = OPTION_FLAG,
+		.opt.flag = &enc_config.pbkdf2,
+	},
 	{
 		.name = "S",
 		.argname = "salt",
@@ -289,30 +303,22 @@ static struct option enc_options[] = {
 	{ NULL },
 };
 
-static void
-show_ciphers(const OBJ_NAME *name, void *arg)
-{
-	static int n;
-
-	if (!islower((unsigned char)*name->name))
-		return;
-
-	fprintf(stderr, " -%-24s%s", name->name, (++n % 3 ? "" : "\n"));
-}
-
 static void
 enc_usage(void)
 {
+	int n = 0;
+
 	fprintf(stderr, "usage: enc -ciphername [-AadePp] [-base64] "
 	    "[-bufsize number] [-debug]\n"
-	    "    [-in file] [-iv IV] [-K key] [-k password]\n"
+	    "    [-in file] [-iter iterations] [-iv IV] [-K key] "
+            "[-k password]\n"
 	    "    [-kfile file] [-md digest] [-none] [-nopad] [-nosalt]\n"
-	    "    [-out file] [-pass arg] [-S salt] [-salt]\n\n");
+	    "    [-out file] [-pass source] [-pbkdf2] [-S salt] [-salt]\n\n");
 	options_usage(enc_options);
 	fprintf(stderr, "\n");
 
 	fprintf(stderr, "Valid ciphername values:\n\n");
-	OBJ_NAME_do_all_sorted(OBJ_NAME_TYPE_CIPHER_METH, show_ciphers, NULL);
+	OBJ_NAME_do_all_sorted(OBJ_NAME_TYPE_CIPHER_METH, show_cipher, &n);
 	fprintf(stderr, "\n");
 }
 
@@ -416,7 +422,7 @@ enc_main(int argc, char **argv)
 		goto end;
 	}
 	if (dgst == NULL) {
-		dgst = EVP_md5();	/* XXX */
+		dgst = EVP_sha256();
 	}
 
 	if (enc_config.bufsize != NULL) {
@@ -604,10 +610,35 @@ enc_main(int argc, char **argv)
 				}
 				sptr = salt;
 			}
+			if (enc_config.pbkdf2 == 1 || enc_config.iter > 0) {
+				/*
+				 * derive key and default iv
+				 * concatenated into a temporary buffer
+				 */
+				unsigned char tmpkeyiv[EVP_MAX_KEY_LENGTH + EVP_MAX_IV_LENGTH];
+				int iklen = EVP_CIPHER_key_length(enc_config.cipher);
+				int ivlen = EVP_CIPHER_iv_length(enc_config.cipher);
+				/* not needed if HASH_UPDATE() is fixed : */
+				int islen = (sptr != NULL ? sizeof(salt) : 0);
+
+				if (enc_config.iter == 0)
+					enc_config.iter = 10000;
+
+				if (!PKCS5_PBKDF2_HMAC(enc_config.keystr,
+					strlen(enc_config.keystr), sptr, islen,
+					enc_config.iter, dgst, iklen+ivlen, tmpkeyiv)) {
+					BIO_printf(bio_err, "PKCS5_PBKDF2_HMAC failed\n");
+					goto end;
+				}
+				/* split and move data back to global buffer */
+				memcpy(key, tmpkeyiv, iklen);
+				memcpy(iv, tmpkeyiv+iklen, ivlen);
+			} else {
+				EVP_BytesToKey(enc_config.cipher, dgst, sptr,
+				    (unsigned char *)enc_config.keystr,
+				    strlen(enc_config.keystr), 1, key, iv);
+			}
 
-			EVP_BytesToKey(enc_config.cipher, dgst, sptr,
-			    (unsigned char *)enc_config.keystr,
-			    strlen(enc_config.keystr), 1, key, iv);
 			/*
 			 * zero the complete buffer or the string passed from
 			 * the command line bug picked up by Larry J. Hughes
@@ -717,13 +748,12 @@ enc_main(int argc, char **argv)
 		BIO_printf(bio_err, "bytes read   :%8ld\n", BIO_number_read(in));
 		BIO_printf(bio_err, "bytes written:%8ld\n", BIO_number_written(out));
 	}
-end:
+ end:
 	ERR_print_errors(bio_err);
 	free(strbuf);
 	free(buff);
 	BIO_free(in);
-	if (out != NULL)
-		BIO_free_all(out);
+	BIO_free_all(out);
 	BIO_free(benc);
 	BIO_free(b64);
 #ifdef ZLIB
