@@ -1,4 +1,4 @@
-/* $OpenBSD: a_bitstr.c,v 1.24 2017/01/29 17:49:22 beck Exp $ */
+/* $OpenBSD: a_bitstr.c,v 1.29 2018/10/20 16:07:09 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -118,10 +118,11 @@ i2c_ASN1_BIT_STRING(ASN1_BIT_STRING *a, unsigned char **pp)
 
 	*(p++) = (unsigned char)bits;
 	d = a->data;
-	memcpy(p, d, len);
-	p += len;
-	if (len > 0)
-		p[-1]&=(0xff << bits);
+	if (len > 0) {
+		memcpy(p, d, len);
+		p += len;
+		p[-1] &= 0xff << bits;
+	}
 	*pp = p;
 	return (ret);
 }
@@ -135,29 +136,34 @@ c2i_ASN1_BIT_STRING(ASN1_BIT_STRING **a, const unsigned char **pp, long len)
 	int i;
 
 	if (len < 1) {
-		i = ASN1_R_STRING_TOO_SHORT;
+		ASN1error(ASN1_R_STRING_TOO_SHORT);
 		goto err;
 	}
 
-	if ((a == NULL) || ((*a) == NULL)) {
+	if (a == NULL || *a == NULL) {
 		if ((ret = ASN1_BIT_STRING_new()) == NULL)
 			return (NULL);
 	} else
-		ret = (*a);
+		ret = *a;
 
 	p = *pp;
 	i = *(p++);
-	/* We do this to preserve the settings.  If we modify
-	 * the settings, via the _set_bit function, we will recalculate
-	 * on output */
-	ret->flags&= ~(ASN1_STRING_FLAG_BITS_LEFT|0x07); /* clear */
-	ret->flags|=(ASN1_STRING_FLAG_BITS_LEFT|(i&0x07)); /* set */
+	if (i > 7) {
+		ASN1error(ASN1_R_INVALID_BIT_STRING_BITS_LEFT);
+		goto err;
+	}
 
-	if (len-- > 1) /* using one because of the bits left byte */
-	{
-		s = malloc(len);
-		if (s == NULL) {
-			i = ERR_R_MALLOC_FAILURE;
+	/*
+	 * We do this to preserve the settings. If we modify the settings,
+	 * via the _set_bit function, we will recalculate on output.
+	 */
+	ret->flags &= ~(ASN1_STRING_FLAG_BITS_LEFT | 0x07); /* clear */
+	ret->flags |= (ASN1_STRING_FLAG_BITS_LEFT | i); /* set */
+
+	/* using one because of the bits left byte */
+	if (len-- > 1) {
+		if ((s = malloc(len)) == NULL) {
+			ASN1error(ERR_R_MALLOC_FAILURE);
 			goto err;
 		}
 		memcpy(s, p, len);
@@ -166,24 +172,25 @@ c2i_ASN1_BIT_STRING(ASN1_BIT_STRING **a, const unsigned char **pp, long len)
 	} else
 		s = NULL;
 
-	ret->length = (int)len;
 	free(ret->data);
 	ret->data = s;
+	ret->length = (int)len;
 	ret->type = V_ASN1_BIT_STRING;
+
 	if (a != NULL)
-		(*a) = ret;
+		*a = ret;
+
 	*pp = p;
+
 	return (ret);
 
-err:
-	ASN1error(i);
-	if ((ret != NULL) && ((a == NULL) || (*a != ret)))
+ err:
+	if (a == NULL || *a != ret)
 		ASN1_BIT_STRING_free(ret);
+
 	return (NULL);
 }
 
-/* These next 2 functions from Goetz Babin-Ebell <babinebell@trustcenter.de>
- */
 int
 ASN1_BIT_STRING_set_bit(ASN1_BIT_STRING *a, int n, int value)
 {
@@ -204,8 +211,7 @@ ASN1_BIT_STRING_set_bit(ASN1_BIT_STRING *a, int n, int value)
 	if ((a->length < (w + 1)) || (a->data == NULL)) {
 		if (!value)
 			return(1); /* Don't need to set */
-		c = OPENSSL_realloc_clean(a->data, a->length, w + 1);
-		if (c == NULL) {
+		if ((c = recallocarray(a->data, a->length, w + 1, 1)) == NULL) {
 			ASN1error(ERR_R_MALLOC_FAILURE);
 			return 0;
 		}
@@ -222,7 +228,7 @@ ASN1_BIT_STRING_set_bit(ASN1_BIT_STRING *a, int n, int value)
 }
 
 int
-ASN1_BIT_STRING_get_bit(ASN1_BIT_STRING *a, int n)
+ASN1_BIT_STRING_get_bit(const ASN1_BIT_STRING *a, int n)
 {
 	int w, v;
 
@@ -240,7 +246,8 @@ ASN1_BIT_STRING_get_bit(ASN1_BIT_STRING *a, int n)
  * 'len' is the length of 'flags'.
  */
 int
-ASN1_BIT_STRING_check(ASN1_BIT_STRING *a, unsigned char *flags, int flags_len)
+ASN1_BIT_STRING_check(const ASN1_BIT_STRING *a, const unsigned char *flags,
+    int flags_len)
 {
 	int i, ok;
 
