@@ -1,4 +1,4 @@
-/* $OpenBSD: ecs_ossl.c,v 1.9.2.2 2018/11/17 18:46:43 tb Exp $ */
+/* $OpenBSD: ecs_ossl.c,v 1.18 2019/01/19 01:12:48 tb Exp $ */
 /*
  * Written by Nils Larsch for the OpenSSL project
  */
@@ -56,6 +56,8 @@
  *
  */
 
+#include <string.h>
+
 #include <openssl/opensslconf.h>
 
 #include <openssl/err.h>
@@ -65,7 +67,9 @@
 #include "bn_lcl.h"
 #include "ecs_locl.h"
 
-static ECDSA_SIG *ecdsa_do_sign(const unsigned char *dgst, int dlen,
+static int ecdsa_prepare_digest(const unsigned char *dgst, int dgst_len,
+    BIGNUM *order, BIGNUM *ret);
+static ECDSA_SIG *ecdsa_do_sign(const unsigned char *dgst, int dgst_len,
     const BIGNUM *, const BIGNUM *, EC_KEY *eckey);
 static int ecdsa_sign_setup(EC_KEY *eckey, BN_CTX *ctx_in, BIGNUM **kinvp,
     BIGNUM **rp);
@@ -85,37 +89,72 @@ ECDSA_OpenSSL(void)
 	return &openssl_ecdsa_meth;
 }
 
+static int
+ecdsa_prepare_digest(const unsigned char *dgst, int dgst_len, BIGNUM *order,
+    BIGNUM *ret)
+{
+	int dgst_bits, order_bits;
+
+	if (!BN_bin2bn(dgst, dgst_len, ret)) {
+		ECDSAerror(ERR_R_BN_LIB);
+		return 0;
+	}
+
+	/* FIPS 186-3 6.4: Use order_bits leftmost bits if digest is too long */
+	dgst_bits = 8 * dgst_len;
+	order_bits = BN_num_bits(order);
+	if (dgst_bits > order_bits) {
+		if (!BN_rshift(ret, ret, dgst_bits - order_bits)) {
+			ECDSAerror(ERR_R_BN_LIB);
+			return 0;
+		}
+	}
+
+	return 1;
+}
+
+int
+ossl_ecdsa_sign(int type, const unsigned char *dgst, int dlen, unsigned char *sig,
+    unsigned int *siglen, const BIGNUM *kinv, const BIGNUM *r, EC_KEY *eckey)
+{
+	ECDSA_SIG *s;
+
+	if ((s = ECDSA_do_sign_ex(dgst, dlen, kinv, r, eckey)) == NULL) {
+		*siglen = 0;
+		return 0;
+	}
+	*siglen = i2d_ECDSA_SIG(s, &sig);
+	ECDSA_SIG_free(s);
+	return 1;
+}
+
 static int
 ecdsa_sign_setup(EC_KEY *eckey, BN_CTX *ctx_in, BIGNUM **kinvp, BIGNUM **rp)
 {
-	BN_CTX   *ctx = NULL;
-	BIGNUM	 *k = NULL, *r = NULL, *order = NULL, *X = NULL;
-	EC_POINT *tmp_point = NULL;
+	BN_CTX *ctx = ctx_in;
+	BIGNUM *k = NULL, *r = NULL, *order = NULL, *X = NULL;
+	EC_POINT *point = NULL;
 	const EC_GROUP *group;
-	int 	 ret = 0;
+	int order_bits, ret = 0;
 
 	if (eckey == NULL || (group = EC_KEY_get0_group(eckey)) == NULL) {
 		ECDSAerror(ERR_R_PASSED_NULL_PARAMETER);
 		return 0;
 	}
 
-	if (ctx_in == NULL) {
+	if (ctx == NULL) {
 		if ((ctx = BN_CTX_new()) == NULL) {
 			ECDSAerror(ERR_R_MALLOC_FAILURE);
 			return 0;
 		}
-	} else
-		ctx = ctx_in;
+	}
 
-	k = BN_new();	/* this value is later returned in *kinvp */
-	r = BN_new();	/* this value is later returned in *rp    */
-	order = BN_new();
-	X = BN_new();
-	if (!k || !r || !order || !X) {
+	if ((k = BN_new()) == NULL || (r = BN_new()) == NULL ||
+	    (order = BN_new()) == NULL || (X = BN_new()) == NULL) {
 		ECDSAerror(ERR_R_MALLOC_FAILURE);
 		goto err;
 	}
-	if ((tmp_point = EC_POINT_new(group)) == NULL) {
+	if ((point = EC_POINT_new(group)) == NULL) {
 		ECDSAerror(ERR_R_EC_LIB);
 		goto err;
 	}
@@ -124,44 +163,60 @@ ecdsa_sign_setup(EC_KEY *eckey, BN_CTX *ctx_in, BIGNUM **kinvp, BIGNUM **rp)
 		goto err;
 	}
 
+	/* Preallocate space. */
+	order_bits = BN_num_bits(order);
+	if (!BN_set_bit(k, order_bits) ||
+	    !BN_set_bit(r, order_bits) ||
+	    !BN_set_bit(X, order_bits))
+		goto err;
+
 	do {
-		/* get random k */
-		do
+		do {
 			if (!BN_rand_range(k, order)) {
-				ECDSAerror(ECDSA_R_RANDOM_NUMBER_GENERATION_FAILED);
+				ECDSAerror(
+				    ECDSA_R_RANDOM_NUMBER_GENERATION_FAILED);
 				goto err;
 			}
-		while (BN_is_zero(k));
+		} while (BN_is_zero(k));
 
-		/* We do not want timing information to leak the length of k,
-		 * so we compute G*k using an equivalent scalar of fixed
-		 * bit-length. */
-		if (!BN_add(k, k, order))
+		/*
+		 * We do not want timing information to leak the length of k,
+		 * so we compute G * k using an equivalent scalar of fixed
+		 * bit-length.
+		 *
+		 * We unconditionally perform both of these additions to prevent
+		 * a small timing information leakage.  We then choose the sum
+		 * that is one bit longer than the order.  This guarantees the
+		 * code path used in the constant time implementations
+		 * elsewhere.
+		 *
+		 * TODO: revisit the BN_copy aiming for a memory access agnostic
+		 * conditional copy.
+		 */
+		if (!BN_add(r, k, order) ||
+		    !BN_add(X, r, order) ||
+		    !BN_copy(k, BN_num_bits(r) > order_bits ? r : X))
 			goto err;
-		if (BN_num_bits(k) <= BN_num_bits(order))
-			if (!BN_add(k, k, order))
-				goto err;
 
 		BN_set_flags(k, BN_FLG_CONSTTIME);
 
-		/* compute r the x-coordinate of generator * k */
-		if (!EC_POINT_mul(group, tmp_point, k, NULL, NULL, ctx)) {
+		/* Compute r, the x-coordinate of G * k. */
+		if (!EC_POINT_mul(group, point, k, NULL, NULL, ctx)) {
 			ECDSAerror(ERR_R_EC_LIB);
 			goto err;
 		}
 		if (EC_METHOD_get_field_type(EC_GROUP_method_of(group)) ==
 		    NID_X9_62_prime_field) {
-			if (!EC_POINT_get_affine_coordinates_GFp(group,
-			    tmp_point, X, NULL, ctx)) {
+			if (!EC_POINT_get_affine_coordinates_GFp(group, point,
+			    X, NULL, ctx)) {
 				ECDSAerror(ERR_R_EC_LIB);
 				goto err;
 			}
 		}
 #ifndef OPENSSL_NO_EC2M
-		else /* NID_X9_62_characteristic_two_field */
-		{
-			if (!EC_POINT_get_affine_coordinates_GF2m(group,
-			    tmp_point, X, NULL, ctx)) {
+		else {	/* NID_X9_62_characteristic_two_field */
+			if (!EC_POINT_get_affine_coordinates_GF2m(group, point,
+			    X, NULL, ctx)) {
 				ECDSAerror(ERR_R_EC_LIB);
 				goto err;
 			}
@@ -173,32 +228,39 @@ ecdsa_sign_setup(EC_KEY *eckey, BN_CTX *ctx_in, BIGNUM **kinvp, BIGNUM **rp)
 		}
 	} while (BN_is_zero(r));
 
-	/* compute the inverse of k */
 	if (!BN_mod_inverse_ct(k, k, order, ctx)) {
 		ECDSAerror(ERR_R_BN_LIB);
 		goto err;
 	}
-	/* clear old values if necessary */
 	BN_clear_free(*rp);
 	BN_clear_free(*kinvp);
-	/* save the pre-computed values  */
 	*rp = r;
 	*kinvp = k;
 	ret = 1;
 
-err:
-	if (!ret) {
+ err:
+	if (ret == 0) {
 		BN_clear_free(k);
 		BN_clear_free(r);
 	}
 	if (ctx_in == NULL)
 		BN_CTX_free(ctx);
 	BN_free(order);
-	EC_POINT_free(tmp_point);
+	EC_POINT_free(point);
 	BN_clear_free(X);
 	return (ret);
 }
 
+/* replace w/ ecdsa_sign_setup() when ECDSA_METHOD gets removed */
+int
+ossl_ecdsa_sign_setup(EC_KEY *eckey, BN_CTX *ctx_in, BIGNUM **kinvp, BIGNUM **rp)
+{
+	ECDSA_DATA *ecdsa;
+
+	if ((ecdsa = ecdsa_check(eckey)) == NULL)
+		return 0;
+	return ecdsa->meth->ecdsa_sign_setup(eckey, ctx_in, kinvp, rp);
+}
 
 static ECDSA_SIG *
 ecdsa_do_sign(const unsigned char *dgst, int dgst_len,
@@ -211,7 +273,7 @@ ecdsa_do_sign(const unsigned char *dgst, int dgst_len,
 	const EC_GROUP *group;
 	ECDSA_SIG  *ret;
 	ECDSA_DATA *ecdsa;
-	int     ok = 0, i;
+	int ok = 0;
 
 	ecdsa = ecdsa_check(eckey);
 	group = EC_KEY_get0_group(eckey);
@@ -222,8 +284,7 @@ ecdsa_do_sign(const unsigned char *dgst, int dgst_len,
 		return NULL;
 	}
 
-	ret = ECDSA_SIG_new();
-	if (!ret) {
+	if ((ret = ECDSA_SIG_new()) == NULL) {
 		ECDSAerror(ERR_R_MALLOC_FAILURE);
 		return NULL;
 	}
@@ -241,21 +302,10 @@ ecdsa_do_sign(const unsigned char *dgst, int dgst_len,
 		ECDSAerror(ERR_R_EC_LIB);
 		goto err;
 	}
-	i = BN_num_bits(order);
-	/* Need to truncate digest if it is too long: first truncate whole
-	 * bytes.
-	 */
-	if (8 * dgst_len > i)
-		dgst_len = (i + 7)/8;
-	if (!BN_bin2bn(dgst, dgst_len, m)) {
-		ECDSAerror(ERR_R_BN_LIB);
+
+	if (!ecdsa_prepare_digest(dgst, dgst_len, order, m))
 		goto err;
-	}
-	/* If still too long truncate remaining bits with a shift */
-	if ((8 * dgst_len > i) && !BN_rshift(m, m, 8 - (i & 0x7))) {
-		ECDSAerror(ERR_R_BN_LIB);
-		goto err;
-	}
+
 	do {
 		if (in_kinv == NULL || in_r == NULL) {
 			if (!ECDSA_sign_setup(eckey, ctx, &kinv, &ret->r)) {
@@ -272,7 +322,9 @@ ecdsa_do_sign(const unsigned char *dgst, int dgst_len,
 		}
 
 		/*
-		 * Compute s = inv(k)(m + xr) mod order.
+		 * Compute:
+		 *
+		 *  s = inv(k)(m + xr) mod order
 		 *
 		 * In order to reduce the possibility of a side-channel attack,
 		 * the following is calculated using a blinding value:
@@ -281,7 +333,7 @@ ecdsa_do_sign(const unsigned char *dgst, int dgst_len,
 		 *
 		 * where b is a random value in the range [1, order-1].
 		 */
-		
+
 		/* Generate b in range [1, order-1]. */
 		if (!BN_sub(range, order, BN_value_one())) {
 			ECDSAerror(ERR_R_BN_LIB);
@@ -342,8 +394,8 @@ ecdsa_do_sign(const unsigned char *dgst, int dgst_len,
 
 	ok = 1;
 
-err:
-	if (!ok) {
+ err:
+	if (ok == 0) {
 		ECDSA_SIG_free(ret);
 		ret = NULL;
 	}
@@ -359,26 +411,62 @@ err:
 	return ret;
 }
 
+/* replace w/ ecdsa_do_sign() when ECDSA_METHOD gets removed */
+ECDSA_SIG *
+ossl_ecdsa_sign_sig(const unsigned char *dgst, int dgst_len,
+    const BIGNUM *in_kinv, const BIGNUM *in_r, EC_KEY *eckey)
+{
+	ECDSA_DATA *ecdsa;
+
+	if ((ecdsa = ecdsa_check(eckey)) == NULL)
+		return NULL;
+	return ecdsa->meth->ecdsa_do_sign(dgst, dgst_len, in_kinv, in_r, eckey);
+}
+
+int
+ossl_ecdsa_verify(int type, const unsigned char *dgst, int dgst_len,
+    const unsigned char *sigbuf, int sig_len, EC_KEY *eckey)
+{
+	ECDSA_SIG *s;
+	unsigned char *der = NULL;
+	const unsigned char *p = sigbuf;
+	int derlen = -1;
+	int ret = -1;
+
+	if ((s = ECDSA_SIG_new()) == NULL)
+		return (ret);
+	if (d2i_ECDSA_SIG(&s, &p, sig_len) == NULL)
+		goto err;
+	/* Ensure signature uses DER and doesn't have trailing garbage */
+	derlen = i2d_ECDSA_SIG(s, &der);
+	if (derlen != sig_len || memcmp(sigbuf, der, derlen))
+		goto err;
+	ret = ECDSA_do_verify(dgst, dgst_len, s, eckey);
+
+ err:
+	freezero(der, derlen);
+	ECDSA_SIG_free(s);
+	return (ret);
+}
+
 static int
 ecdsa_do_verify(const unsigned char *dgst, int dgst_len, const ECDSA_SIG *sig,
     EC_KEY *eckey)
 {
-	int ret = -1, i;
-	BN_CTX   *ctx;
-	BIGNUM   *order, *u1, *u2, *m, *X;
+	BN_CTX *ctx;
+	BIGNUM *order, *u1, *u2, *m, *X;
 	EC_POINT *point = NULL;
 	const EC_GROUP *group;
 	const EC_POINT *pub_key;
+	int ret = -1;
 
-	/* check input values */
 	if (eckey == NULL || (group = EC_KEY_get0_group(eckey)) == NULL ||
 	    (pub_key = EC_KEY_get0_public_key(eckey)) == NULL || sig == NULL) {
 		ECDSAerror(ECDSA_R_MISSING_PARAMETERS);
 		return -1;
 	}
 
-	ctx = BN_CTX_new();
-	if (!ctx) {
+	if ((ctx = BN_CTX_new()) == NULL) {
 		ECDSAerror(ERR_R_MALLOC_FAILURE);
 		return -1;
 	}
@@ -388,7 +476,7 @@ ecdsa_do_verify(const unsigned char *dgst, int dgst_len, const ECDSA_SIG *sig,
 	u2 = BN_CTX_get(ctx);
 	m = BN_CTX_get(ctx);
 	X = BN_CTX_get(ctx);
-	if (!X) {
+	if (X == NULL) {
 		ECDSAerror(ERR_R_BN_LIB);
 		goto err;
 	}
@@ -398,45 +486,33 @@ ecdsa_do_verify(const unsigned char *dgst, int dgst_len, const ECDSA_SIG *sig,
 		goto err;
 	}
 
-	if (BN_is_zero(sig->r)          || BN_is_negative(sig->r) ||
-	    BN_ucmp(sig->r, order) >= 0 || BN_is_zero(sig->s)  ||
-	    BN_is_negative(sig->s)      || BN_ucmp(sig->s, order) >= 0) {
+	/* Verify that r and s are in the range [1, order-1]. */
+	if (BN_is_zero(sig->r) || BN_is_negative(sig->r) ||
+	    BN_ucmp(sig->r, order) >= 0 ||
+	    BN_is_zero(sig->s) || BN_is_negative(sig->s) ||
+	    BN_ucmp(sig->s, order) >= 0) {
 		ECDSAerror(ECDSA_R_BAD_SIGNATURE);
-		ret = 0;	/* signature is invalid */
+		ret = 0;
 		goto err;
 	}
-	/* calculate tmp1 = inv(S) mod order */
-	if (!BN_mod_inverse_ct(u2, sig->s, order, ctx)) {
+
+	if (!ecdsa_prepare_digest(dgst, dgst_len, order, m))
+		goto err;
+
+	if (!BN_mod_inverse_ct(u2, sig->s, order, ctx)) {	/* w = inv(s) */
 		ECDSAerror(ERR_R_BN_LIB);
 		goto err;
 	}
-	/* digest -> m */
-	i = BN_num_bits(order);
-	/* Need to truncate digest if it is too long: first truncate whole
-	 * bytes.
-	 */
-	if (8 * dgst_len > i)
-		dgst_len = (i + 7)/8;
-	if (!BN_bin2bn(dgst, dgst_len, m)) {
+	if (!BN_mod_mul(u1, m, u2, order, ctx)) {		/* u1 = mw */
 		ECDSAerror(ERR_R_BN_LIB);
 		goto err;
 	}
-	/* If still too long truncate remaining bits with a shift */
-	if ((8 * dgst_len > i) && !BN_rshift(m, m, 8 - (i & 0x7))) {
-		ECDSAerror(ERR_R_BN_LIB);
-		goto err;
-	}
-	/* u1 = m * tmp mod order */
-	if (!BN_mod_mul(u1, m, u2, order, ctx)) {
-		ECDSAerror(ERR_R_BN_LIB);
-		goto err;
-	}
-	/* u2 = r * w mod q */
-	if (!BN_mod_mul(u2, sig->r, u2, order, ctx)) {
+	if (!BN_mod_mul(u2, sig->r, u2, order, ctx)) {		/* u2 = rw */
 		ECDSAerror(ERR_R_BN_LIB);
 		goto err;
 	}
 
+	/* Compute the x-coordinate of G * u1 + pub_key * u2. */
 	if ((point = EC_POINT_new(group)) == NULL) {
 		ECDSAerror(ERR_R_MALLOC_FAILURE);
 		goto err;
@@ -447,17 +523,16 @@ ecdsa_do_verify(const unsigned char *dgst, int dgst_len, const ECDSA_SIG *sig,
 	}
 	if (EC_METHOD_get_field_type(EC_GROUP_method_of(group)) ==
 	    NID_X9_62_prime_field) {
-		if (!EC_POINT_get_affine_coordinates_GFp(group,
-		    point, X, NULL, ctx)) {
+		if (!EC_POINT_get_affine_coordinates_GFp(group, point, X, NULL,
+		    ctx)) {
 			ECDSAerror(ERR_R_EC_LIB);
 			goto err;
 		}
 	}
 #ifndef OPENSSL_NO_EC2M
-	else /* NID_X9_62_characteristic_two_field */
-	{
-		if (!EC_POINT_get_affine_coordinates_GF2m(group,
-		    point, X, NULL, ctx)) {
+	else { /* NID_X9_62_characteristic_two_field */
+		if (!EC_POINT_get_affine_coordinates_GF2m(group, point, X, NULL,
+		    ctx)) {
 			ECDSAerror(ERR_R_EC_LIB);
 			goto err;
 		}
@@ -467,12 +542,25 @@ ecdsa_do_verify(const unsigned char *dgst, int dgst_len, const ECDSA_SIG *sig,
 		ECDSAerror(ERR_R_BN_LIB);
 		goto err;
 	}
-	/*  if the signature is correct u1 is equal to sig->r */
+
+	/* If the signature is correct, the x-coordinate is equal to sig->r. */
 	ret = (BN_ucmp(u1, sig->r) == 0);
 
-err:
+ err:
 	BN_CTX_end(ctx);
 	BN_CTX_free(ctx);
 	EC_POINT_free(point);
 	return ret;
 }
+
+/* replace w/ ecdsa_do_verify() when ECDSA_METHOD gets removed */
+int
+ossl_ecdsa_verify_sig(const unsigned char *dgst, int dgst_len,
+    const ECDSA_SIG *sig, EC_KEY *eckey)
+{
+	ECDSA_DATA *ecdsa;
+
+	if ((ecdsa = ecdsa_check(eckey)) == NULL)
+		return 0;
+	return ecdsa->meth->ecdsa_do_verify(dgst, dgst_len, sig, eckey);
+}
