@@ -1,4 +1,4 @@
-/* $OpenBSD: tls_config.c,v 1.53 2018/11/29 14:24:23 tedu Exp $ */
+/* $OpenBSD: tls_config.c,v 1.56 2019/04/04 15:09:09 jsing Exp $ */
 /*
  * Copyright (c) 2014 Joel Sing <jsing@openbsd.org>
  *
@@ -24,6 +24,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <pthread.h>
 #include <stdlib.h>
 #include <unistd.h>
 
@@ -97,12 +98,15 @@ tls_config_new_internal(void)
 	if ((config = calloc(1, sizeof(*config))) == NULL)
 		return (NULL);
 
-	if ((config->keypair = tls_keypair_new()) == NULL)
+	if (pthread_mutex_init(&config->mutex, NULL) != 0)
 		goto err;
 
 	config->refcount = 1;
 	config->session_fd = -1;
 
+	if ((config->keypair = tls_keypair_new()) == NULL)
+		goto err;
+
 	/*
 	 * Default configuration.
 	 */
@@ -153,11 +157,16 @@ void
 tls_config_free(struct tls_config *config)
 {
 	struct tls_keypair *kp, *nkp;
+	int refcount;
 
 	if (config == NULL)
 		return;
 
-	if (--config->refcount > 0)
+	pthread_mutex_lock(&config->mutex);
+	refcount = --config->refcount;
+	pthread_mutex_unlock(&config->mutex);
+
+	if (refcount > 0)
 		return;
 
 	for (kp = config->keypair; kp != NULL; kp = nkp) {
@@ -623,8 +632,6 @@ tls_config_set_keypair_file_internal(struct tls_config *config,
 		return (-1);
 	if (tls_config_set_key_file(config, key_file) != 0)
 		return (-1);
-	if (tls_config_set_key_file(config, key_file) != 0)
-		return (-1);
 	if (ocsp_file != NULL &&
 	    tls_config_set_ocsp_staple_file(config, ocsp_file) != 0)
 		return (-1);
